#!/bin/sh
echo "Quality of Service"
eval `flash get WAN1_QOS_ENABLED`
eval `flash get WAN1_QOS_TBL_NUM`
if [ "$_EZ_QOS_" = "y" ]; then
eval `flash get EZ_QOS_ENABLED`
fi
if [ "$WAN1_QOS_ENABLED" = 0 ] || [ "$EZ_QOS_ENABLED" = 1 ]; then
echo 0, > /proc/qos
exit 0
fi
. /bin/scriptlib.sh
TMP1=/var/qos1
TMP2=/var/qos2
T1=`ip route | grep via | grep -v grep | cut -f 2 | grep $_WAN_IF_ | grep -v grep | cut -f 2-6 -d" "`
addQueueNode()
{
UBValue=`echo $1 | cut -d ',' -f 1` #upload bandwidth
DBValue=`echo $1 | cut -d ',' -f 3` #download bandwidth
LIP=`echo $1 | cut -d ',' -f 6` #local ip range
RIP=`echo $1 | cut -d ',' -f 8` #remote ip range
TRAType=`echo $1 | cut -d ',' -f 9` #traffic type HTTP,FTP...
PROType=`echo $1 | cut -d ',' -f 10` #protocol type tcp or udp
SPort=`echo $1 | cut -d ',' -f 11 | cut -d '-' -f 1` #source port range
DPort=`echo $1 | cut -d ',' -f 12 | cut -d '-' -f 1` #destination port range
if [ "$DBValue" != "0" ]; then #download
BWType=`echo $1 | cut -d ',' -f 4` #maximum and guarantee
elif [ "$UBValue" != "0" ]; then #upload
BWType=`echo $1 | cut -d ',' -f 2` #maximum and guarantee
fi
classid=`expr 10 + $2`
handle=`expr 50 \* $3`
handle=`expr $handle + $2`
filterD=`expr 100 \* $3`
filterD=`expr $filterD + $2`
filterU=`expr 1000 \* $3`
filterU=`expr $filterU + $2`
if [ "$BWType" = "0" ]; then #Guarantee
if [ "$UBValue" != "0" ]; then #upload
tc class add dev $4 parent $3:1 classid $3:$classid htb rate "$UBValue"kbit ceil "$5"kbit prio $2 quantum 1500
tc qdisc add dev $4 parent $3:$classid handle $handle: sfq
tc filter add dev $4 parent $3:  protocol ip prio $2 handle $filterU fw flowid $3:$classid
elif [ "$DBValue" != "0" ]; then #download
tc class add dev br0 parent 8888:8888 classid 8888:$classid htb rate "$DBValue"kbit ceil "$6"kbit prio $2 quantum 1500
tc qdisc add dev br0 parent 8888:$classid handle $handle: sfq
tc filter add dev br0 parent 8888:  protocol ip prio $2 handle $filterD fw flowid 8888:$classid
fi
elif [ "$BWType" = "1" ]; then #maximum
if [ "$UBValue" != "0" ]; then #upload
tc class add dev $4 parent $3:1 classid $3:$classid htb rate 1kbit ceil "$UBValue"kbit prio $2 quantum 1500
tc qdisc add dev $4 parent $3:$classid handle $handle: sfq
tc filter add dev $4 parent $3:  protocol ip prio $2 handle $filterU fw flowid $3:$classid
elif [ "$DBValue" != "0" ]; then #download
tc class add dev br0 parent 8888:8888 classid 8888:$classid htb rate 1kbit ceil "$DBValue"kbit prio $2 quantum 1500
tc qdisc add dev br0 parent 8888:$classid handle $handle: sfq
tc filter add dev br0 parent 8888:  protocol ip prio $2 handle $filterD fw flowid 8888:$classid
fi
fi
if [ "$DBValue" != "0" ]; then #download
COMMAND="iptables -A POSTROUTING -t mangle "
elif [ "$UBValue" != "0" ]; then #upload
COMMAND="iptables -A PREROUTING -t mangle "
fi
if [ "$LIP" != "0.0.0.0-0.0.0.0" ]; then
COMMAND=$COMMAND"-m iprange --src-range $LIP "
if [ "$RIP" != "0.0.0.0-0.0.0.0" ]; then
COMMAND=$COMMAND"--dst-range $RIP "
fi
else
if [ "$RIP" != "0.0.0.0-0.0.0.0" ]; then
COMMAND=$COMMAND"-m iprange --dst-range $RIP "
fi
fi
if [ "$TRAType" = "SMTP" ]; then
COMMAND=$COMMAND"-p tcp --destination-port 25 "
elif [ "$TRAType" = "HTTP" ]; then
if [ "$DBValue" != "0" ]; then #download
COMMAND=$COMMAND"-p tcp --source-port 80 "
elif [ "$UBValue" != "0" ]; then #upload
COMMAND=$COMMAND"-p tcp --destination-port 80 "
fi
elif [ "$TRAType" = "POP3" ]; then
if [ "$DBValue" != "0" ]; then #download
COMMAND=$COMMAND"-p tcp --source-port 110 "
fi
elif [ "$TRAType" = "FTP" ]; then
if [ "$DBValue" != "0" ]; then #download
COMMAND=$COMMAND"-p tcp --source-port 20 "
elif [ "$UBValue" != "0" ]; then #upload
COMMAND=$COMMAND"-p tcp --destination-port 20 "
fi
else
if [ "$PROType" = "0" ]; then #tcp
COMMAND=$COMMAND" -p tcp "
elif [ "$PROType" = "1" ]; then #udp
COMMAND=$COMMAND" -p udp "
fi
if [ "$SPort" != "0:0" ]; then
COMMAND=$COMMAND"--source-port $SPort "
fi
if [ "$DPort" != "0:0" ]; then
COMMAND=$COMMAND"--destination-port $DPort "
fi
fi
if [ "$DBValue" != "0" ]; then #download
RETURN=$COMMAND"-o br0 -j RETURN"
COMMAND=$COMMAND"-o br0 -j MARK --set-mark $filterD"
ip rule del fwmark $filterD lookup T$3 2>/dev/null
ip rule add fwmark $filterD lookup T$3
elif [ "$UBValue" != "0" ]; then #upload
RETURN=$COMMAND"-i br0 -j RETURN"
COMMAND=$COMMAND"-i br0 -j MARK --set-mark $filterU"
ip rule del fwmark $filterU lookup T$3 2>/dev/null
ip rule add fwmark $filterU lookup T$3
fi
$COMMAND
$RETURN
echo "$COMMAND"
echo "$RETURN"
}
addQueueRoot()
{
BWMAX=60000
DMAX=`/bin/flash get MAX_DOWNLOAD_BANDWIDTH | cut -d '=' -f 2`
UMAX=`/bin/flash get MAX_UPLOAD_BANDWIDTH | cut -d '=' -f 2`
NDMAX=0
NUMAX=0
DALLGA=0
UALLGA=0
TBL_NUM=`/bin/flash get WAN$1_QOS_TBL_NUM | cut -d '=' -f 2`
DGBW=$BWMAX
UGBW=$BWMAX
if [ "$DMAX" != "0" ]; then
DGBW=$DMAX
fi
if [ "$UMAX" != "0" ]; then
UGBW=$UMAX
fi
i=1
while [ $i -le $TBL_NUM ]
do
QOS_TBL=`/bin/flash get WAN$1_QOS_TBL | grep TBL$i | cut -d '=' -f 2`
UV=`echo $QOS_TBL | cut -d ',' -f 1` #download bandwidth
DV=`echo $QOS_TBL | cut -d ',' -f 3` #upload bandwidth
if [ "$DV" != "0" ]; then #download
BT=`echo $QOS_TBL | cut -d ',' -f 4` #maximum and guarantee
if [ "$BT" = "0" ]; then
DGBW=`expr $DGBW - $DV`
DALLGA=`expr $DALLGA + $DV`
fi
elif [ "$UV" != "0" ]; then #upload
BT=`echo $QOS_TBL | cut -d ',' -f 2` #maximum and guarantee
if [ "$BT" = "0" ]; then
UGBW=`expr $UGBW - $UV`
UALLGA=`expr $UALLGA + $UV`
fi
fi
i=`expr $i + 1`
done
TEMP=`expr $DGBW / 10`
DGBW=`expr $DGBW - $TEMP`
TEMP=`expr $UGBW / 10`
UGBW=`expr $UGBW - $TEMP`
DGBW=1
UGBW=1
tc qdisc del dev br0 root 2>/dev/null
tc qdisc add dev br0 root handle 8888: htb default 9999
if [ "$DMAX" = "0" ]; then
tc class add dev br0 parent 8888: classid 8888:8888 htb rate "$BWMAX"kbit ceil "$BWMAX"kbit quantum 1500
tc class add dev br0 parent 8888:8888 classid 8888:9999 htb rate 1kbit ceil "$BWMAX"kbit quantum 1500
NDMAX=$BWMAX
else
tc class add dev br0 parent 8888: classid 8888:8888 htb rate "$DMAX"kbit ceil "$DMAX"kbit quantum 1500
tc class add dev br0 parent 8888:8888 classid 8888:9999 htb rate 1kbit ceil "$DMAX"kbit quantum 1500
NDMAX=$DMAX
fi
tc qdisc add dev br0 parent 8888:9999 handle 999: sfq
tc qdisc del dev $2 root 2>/dev/null
tc qdisc add dev $2 root handle $1: htb default 5
if [ "$UMAX" = "0" ]; then
tc class add dev $2 parent $1: classid $1:1 htb rate "$BWMAX"kbit ceil "$BWMAX"kbit quantum 1500
tc class add dev $2 parent $1:1 classid $1:5 htb rate 1kbit ceil "$BWMAX"kbit quantum 1500
NUMAX=$BWMAX
else
tc class add dev $2 parent $1: classid $1:1 htb rate "$UMAX"kbit ceil "$UMAX"kbit quantum 1500
tc class add dev $2 parent $1:1 classid $1:5 htb rate 1kbit ceil "$UMAX"kbit quantum 1500
NUMAX=$UMAX
fi
tc qdisc add dev $2 parent $1:5 handle 5: sfq
i=1
while [ $i -le $TBL_NUM ]
do
QOS_TBL=`/bin/flash get WAN$1_QOS_TBL | grep TBL$i | cut -d '=' -f 2`
addQueueNode $QOS_TBL $i $1 $2 $NUMAX $NDMAX
i=`expr $i + 1`
done
}
if [ "$OP_MODE" != 0 ]; then
if [ "$_USB_8712_" = "y" ]; then
WAN1=wlan1
else 
WAN1=wlan0
fi
elif [ "$WAN_MODE" = 2 ]; then
WAN1=ppp0
WAN2=ppp1
elif [ "$WAN_MODE" = 3 ] || [ "$WAN_MODE" = 6 ]; then
WAN1=ppp0
else
WAN1=eth1
fi
if [ "$WAN_MODE" = "2" ]; then
WANLIST="1 2"
else
WANLIST="1"
fi
for wl in $WANLIST
do
eval enabled=\$WAN"$wl"_QOS_ENABLED
if [ "$enabled" = "1" ]; then #QoS & iQoS conflict
ip rule | grep T$wl | grep fwmark | grep -v grep > $TMP1
while [ -s $TMP1 ]
do
TMP2=`head -n 1 $TMP1`
mnum=`echo $TMP2 | cut -f 5 -d " "`
ip rule del fwmark $mnum
ip rule | grep T$wl | grep fwmark | grep -v grep > $TMP1
done
eval WAN=\$WAN$wl
tc qdisc del dev $WAN root 2>/dev/null
rmmod sw_nat 2>/dev/null
addQueueRoot $wl $WAN
fi
done
rm -f $TMP1
rm -f $TMP2
echo 1, > /proc/qos
