#!/bin/sh
. /bin/scriptlib.sh
SCRIPT_TITLE="Intrusion"
WAN_INTERFACE1="`wan_interface 1`"
WAN_INTERFACE2="`wan_interface 2`"
WAN_INTERFACE3="`wan_interface 3`"
WAN_INTERFACER="`wan_interface R`"
WAN_IP1="`ip_of_interface $WAN_INTERFACE1`"
WAN_IP2="`ip_of_interface $WAN_INTERFACE2`"
WAN_IP3="`ip_of_interface $WAN_INTERFACE3`"
WAN_IPR="`ip_of_interface $WAN_INTERFACER`"
WAN_MTU1="`wan_mtu 1`"
WAN_MTU2="`wan_mtu 2`"
WAN_MTU3="`wan_mtu 3`"
debug_print "Cleaning previous status."
if [ -f /var/clean_intrusion.sh ]; then
chmod 777 /var/clean_intrusion.sh
/var/clean_intrusion.sh 2> /dev/null
rm -rf /var/clean_intrusion.sh
fi
debug_print "Checking whether Firewall is enabled."
if [ "$FIREWALL_ENABLE" = "0" ]; then
debug_print "Firewall is disabled, exit."
if [ "$_KERNEL_VERSION_26_" = "y" ]; then
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p icmp --icmp-type echo-request -j ACCEPT"
fi
exit
else
debug_print "Firewall is enabled."
fi
debug_print "Checking whether Ping of Death is enabled."
if [ "$POD_ENABLED" = "0" ]; then
debug_print "Ping of Death is disabled."
else
debug_print "Ping of Death is enabled."
case "$POD_TIME" in
"0") TIME="sec";;
"1") TIME="minute";;
"2") TIME="hour";;
esac
debug_print_and_execute "iptables -N ping-of-death"
debug_print_and_execute "iptables -A ping-of-death -m limit --limit $POD_PACK/$TIME --limit-burst $POD_BUR -j RETURN"
debug_print_and_execute "iptables -A ping-of-death -j REJECT"
debug_print_and_execute "iptables -I INPUT -p icmp --icmp-type echo-request -m state --state NEW -j ping-of-death"
echo "iptables -D INPUT -p icmp --icmp-type echo-request -m state --state NEW -j ping-of-death" >> /var/clean_intrusion.sh
echo "iptables -F ping-of-death" >> /var/clean_intrusion.sh
echo "iptables -X ping-of-death" >> /var/clean_intrusion.sh
fi
if [ "$_DIS_PING_INDEP_" != "y" ];then
debug_print "Checking whether Discard Ping from WAN is enabled."
if [ "$PIN_ENABLED" = "0" ]; then
debug_print "Discard Ping from WAN is disabled."
if [ "$_KERNEL_VERSION_26_" = "y" ]; then
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p icmp --icmp-type echo-request -j ACCEPT"
echo "iptables -A INPUT -i $WAN_INTERFACE1 -p icmp --icmp-type echo-request -j ACCEPT" >> /var/clean_intrusion.sh
fi
else
debug_print "Discard Ping from WAN is enabled."
if [ "$_KERNEL_VERSION_26_" != "y" ]; then
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p icmp --icmp-type echo-request -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p icmp --icmp-type echo-request -j DROP" >> /var/clean_intrusion.sh
fi
fi
fi
debug_print "Checking whether Port Scan is enabled."
if [ "$SCAN_ENABLED" = "0" ]; then
debug_print "Port Scan is disabled."
else
debug_print "Port Scan is enabled."
i=0;
TMP=a
SCAN_TMP=`echo $SCAN_NUM | cut -f2 -d=`
while [ "$TMP" != "" ]; do
i=`expr $i + 1`
OLD=$TMP
TMP=`echo $SCAN_TMP | cut -f$i -d ','`
if [ "$TMP" != "$OLD" ]; then
case "$TMP" in
"nmap")
debug_print "NMAP FIN/URG/PSH"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL FIN,URG,PSH -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL FIN,URG,PSH -j DROP" >> /var/clean_intrusion.sh
;;
"xmastree")
debug_print "Xmas Tree"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL ALL -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL ALL -j DROP" >> /var/clean_intrusion.sh
;;
"axmastree")
debug_print "Another Xmas Tree"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL SYN,RST,ACK,FIN,URG -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL SYN,RST,ACK,FIN,URG -j DROP" >> /var/clean_intrusion.sh
;;
"nullscan")
debug_print "Null Scan"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL NONE -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags ALL NONE -j DROP" >> /var/clean_intrusion.sh
;;
"synrst")
debug_print "SYN/RST"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags SYN,RST SYN,RST -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags SYN,RST SYN,RST -j DROP" >> /var/clean_intrusion.sh
;;
"synfin")
debug_print "SYN/FIN"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags SYN,FIN SYN,FIN -j DROP"
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --tcp-flags SYN,FIN SYN,FIN -j DROP" >> /var/clean_intrusion.sh
;;
"synouport")
debug_print "SYN unreachable port"
debug_print_and_execute "iptables -A OUTPUT -p tcp --tcp-flags RST RST -j DROP"
debug_print_and_execute "iptables -A INPUT -i $WAN_INTERFACE1 -p tcp --dport 53 -j DROP"
echo "iptables -D OUTPUT -p tcp --tcp-flags RST RST -j DROP" >> /var/clean_intrusion.sh
echo "iptables -D INPUT -i $WAN_INTERFACE1 -p tcp --dport 53 -j DROP" >> /var/clean_intrusion.sh
;;
esac
fi
done
fi
debug_print "Checking whether Syn-Flood is enabled."
if [ "$SYN_ENABLED" = "0" ]; then
debug_print "Syn-Flood is disabled."
else
debug_print "Syn-Flood is enabled."
case "$SYN_TIME" in
"0") TIME="sec";;
"1") TIME="minute";;
"2") TIME="hour";;
esac
debug_print_and_execute "iptables -N syn-flood"
debug_print_and_execute "iptables -A syn-flood -p tcp --syn -m limit --limit $SYN_PACK/$TIME --limit-burst $SYN_BUR -j RETURN"
debug_print_and_execute "iptables -A syn-flood -p tcp -j REJECT --reject-with tcp-reset"
debug_print_and_execute "iptables -A INPUT -p tcp -m state --state NEW -j syn-flood"
echo "iptables -D INPUT -p tcp -m state --state NEW -j syn-flood" >> /var/clean_intrusion.sh
echo "iptables -F syn-flood" >> /var/clean_intrusion.sh
echo "iptables -X syn-flood" >> /var/clean_intrusion.sh
fi
debug_print "Exit."
