#!/bin/sh
. /bin/scriptlib.sh
SCRIPT_TITLE="Port Forwarding"
WAN_INTERFACE1="`wan_interface 1`"
WAN_INTERFACE2="`wan_interface 2`"
WAN_INTERFACE3="`wan_interface 3`"
WAN_INTERFACER="`wan_interface R`"
WAN_IP1="`ip_of_interface $WAN_INTERFACE1`"
WAN_IP2="`ip_of_interface $WAN_INTERFACE2`"
WAN_IP3="`ip_of_interface $WAN_INTERFACE3`"
WAN_IPR="`ip_of_interface $WAN_INTERFACER`"
LAN_INTERFACE="`lan_interface`"
LAN_IP="`ip_of_interface $LAN_INTERFACE`"
debug_print "Cleaning previous status."
if [ -f /var/clean_portforwarding.sh ]; then
/var/clean_portforwarding.sh
rm -rf /var/clean_portforwarding.sh
fi
debug_print "Checking whether NAT is enabled."
if [ "$NAT_ENABLE" = "0" ]; then
debug_print "NAT is disabled, exit."
exit 0
else
debug_print "NAT is enabled."
fi
debug_print "Checking whether Port Forwarding is enabled."
if [ "$PORTFW_ENABLED" = "0" ]; then
debug_print "Port Forwarding is disable, exit."
exit 0
else
debug_print "Port Forwarding is enabled."
fi
debug_print "Setting Port Forwarding."
debug_print_and_execute "iptables -t nat -N portfw_SNAT"
debug_print_and_execute "iptables -t nat -A POSTROUTING -j portfw_SNAT"
debug_print_and_execute "iptables -t nat -N portfw_DNAT"
debug_print_and_execute "iptables -t nat -A PREROUTING -j portfw_DNAT"
num=1
while [ $num -le $PORTFW_TBL_NUM ]; do
eval "port_ip="\$PORTFW_TBL$num""
ip=`echo $port_ip | cut -f1 -d,`
port1=`echo $port_ip | cut -f2 -d,`
tmp_port=`echo $port_ip | cut -f3 -d,`
port2=`echo $tmp_port | cut -f2 -d ' '`
protocol=`echo $port_ip | cut -f4 -d,`
session=`echo $port_ip | cut -f5 -d,`
num=`expr $num + 1`
WAN_IP=""
if [ "$session" = "1" ] && [ "$WAN_IP2" != "0.0.0.0" ]; then
WAN_IP=$WAN_IP" "$WAN_IP2
elif [ "$WAN_IP1" != "0.0.0.0" ]; then
WAN_IP=$WAN_IP" "$WAN_IP1
fi
if [ "$_DUALL_WAN_ACCESS_" = "y" ] && [ "$DUALL_ACCESS_ENABLE" = "1" ] && [ "$WAN_IPR" != "0.0.0.0" ]; then
WAN_IP=$WAN_IP" "$WAN_IPR
fi
debug_print "Interfaces:$WAN_IP."
snat_done=0
for WAN in $WAN_IP; do
if [ "$snat_done" = "0" ]; then
debug_print "Set SNAT."
if [ "$_KERNEL_VERSION_26_" = "y" ]; then			
if [ "$port1" = "$port2" ]; then
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p TCP -s $LAN_IP/$SUBNET_MASK --dport $port1 -d $ip -j SNAT --to $LAN_IP"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p UDP -s $LAN_IP/$SUBNET_MASK --dport $port1 -d $ip -j SNAT --to $LAN_IP"
fi
else
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p TCP -s $LAN_IP/$SUBNET_MASK -m multiport --dports $port1:$port2 -d $ip -j SNAT --to $LAN_IP"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p UDP -s $LAN_IP/$SUBNET_MASK -m multiport --dports $port1:$port2 -d $ip  -j SNAT --to $LAN_IP"
fi
fi
if [ "$port1" -le "1723" ] && [ "$port2" -ge "1723" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p 47 -s $LAN_IP/$SUBNET_MASK -d $ip -j SNAT --to $LAN_IP"
fi				
else			
if [ "$port1" = "$port2" ]; then
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p tcp -s $LAN_IP/$SUBNET_MASK -d $ip -m mport --dports $port1 -j SNAT --to-source $LAN_IP"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p udp -s $LAN_IP/$SUBNET_MASK -d $ip -m mport --dports $port1 -j SNAT --to-source $LAN_IP"
fi
else
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p tcp -s $LAN_IP/$SUBNET_MASK -d $ip -m mport --dports $port1:$port2 -j SNAT --to-source $LAN_IP"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p udp -s $LAN_IP/$SUBNET_MASK -d $ip -m mport --dports $port1:$port2 -j SNAT --to-source $LAN_IP"
fi
fi
if [ "$port1" -le "1723" ] && [ "$port2" -ge "1723" ]; then
debug_print_and_execute "iptables -A portfw_SNAT -t nat -p 47 -s $LAN_IP/$SUBNET_MASK -d $ip -j SNAT --to-source $LAN_IP"
fi			
fi
snat_done=1
fi
debug_print "Set DNAT for $ip($port1:$port2)."
if [ "$_KERNEL_VERSION_26_" = "y" ]; then
if [ "$port1" = "$port2" ]; then
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p TCP --dport $port1 -d $WAN -j DNAT --to $ip"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p UDP --dport $port1 -d $WAN -j DNAT --to $ip"
fi
else
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p TCP -m multiport --dports $port1:$port2 -d $WAN -j DNAT --to $ip"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p UDP -m multiport --dports $port1:$port2 -d $WAN -j DNAT --to $ip"
fi
fi		
if [ "$port1" -le "1723" ] && [ "$port2" -ge "1723" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p 47 -d $WAN -j DNAT --to $ip"
fi	
else
if [ "$port1" = "$port2" ]; then
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p tcp -d $WAN -m mport --dports $port1 -j DNAT --to-destination $ip"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p udp -d $WAN -m mport --dports $port1 -j DNAT --to-destination $ip"
fi
else
if [ "$protocol" = "1" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p tcp -d $WAN -m mport --dports $port1:$port2 -j DNAT --to-destination $ip"
fi
if [ "$protocol" = "2" ] || [ "$protocol" = "3" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p udp -d $WAN -m mport --dports $port1:$port2 -j DNAT --to-destination $ip"
fi
fi		
if [ "$port1" -le "1723" ] && [ "$port2" -ge "1723" ]; then
debug_print_and_execute "iptables -A portfw_DNAT -t nat -p 47 -d $WAN -j DNAT --to-destination $ip"
fi
fi
done
done
debug_print "Storing cleaning information."
echo "iptables -t nat -D POSTROUTING -j portfw_SNAT" >> /var/clean_portforwarding.sh
echo "iptables -t nat -D PREROUTING -j portfw_DNAT" >> /var/clean_portforwarding.sh
echo "iptables -t nat -F portfw_SNAT" >> /var/clean_portforwarding.sh
echo "iptables -t nat -F portfw_DNAT" >> /var/clean_portforwarding.sh
echo "iptables -t nat -X portfw_SNAT" >> /var/clean_portforwarding.sh
echo "iptables -t nat -X portfw_DNAT" >> /var/clean_portforwarding.sh
chmod 777 /var/clean_portforwarding.sh
debug_print "Exit."
